@@ -14,264 +14,313 @@
  * limitations under the License.
  */
 
+use clap::Parser;
 use grex::RegExpBuilder;
 use itertools::Itertools;
 use std::io::{BufRead, Error, ErrorKind, Read};
 use std::path::PathBuf;
-use structopt::clap::AppSettings::{AllowLeadingHyphen, ColoredHelp};
-use structopt::StructOpt;
 
-#[derive(StructOpt)]
-#[structopt(
+#[derive(Parser)]
+#[clap(
     author = "© 2019-today Peter M. Stahl <pemistahl@gmail.com>",
     about = "Licensed under the Apache License, Version 2.0\n\
              Downloadable from https://crates.io/crates/grex\n\
              Source code at https://github.com/pemistahl/grex\n\n\
              grex generates regular expressions from user-provided test cases.",
-    version_short = "v",
-    global_settings = &[AllowLeadingHyphen, ColoredHelp]
+    version,
+    allow_hyphen_values = true,
+    mut_arg("help", |help| help.help_heading("MISCELLANEOUS OPTIONS")),
+    mut_arg("version", |version| version.short('v').help_heading("MISCELLANEOUS OPTIONS"))
 )]
 struct Cli {
     // --------------------
-    // ARGS
+    // INPUT
     // --------------------
-    #[structopt(
+    /// One or more test cases separated by blank space
+    #[clap(
         value_name = "INPUT",
-        required_unless = "file",
+        required_unless_present = "file",
         conflicts_with = "file",
-        help = "One or more test cases separated by blank space"
+        value_parser,
+        help_heading = "INPUT",
+        display_order = 1
     )]
     input: Vec<String>,
 
-    // --------------------
-    // FLAGS
-    // --------------------
-    #[structopt(
-        name = "digits",
-        short,
-        long,
-        help = "Converts any Unicode decimal digit to \\d",
-        long_help = "Converts any Unicode decimal digit to \\d.\n\n\
-                     Takes precedence over --words if both are set.\n\
-                     Decimal digits are converted to \\d, remaining word characters to \\w.\n\n\
-                     Takes precedence over --non-spaces if both are set.\n\
-                     Decimal digits are converted to \\d, remaining non-space characters to \\S.",
-        display_order = 1
-    )]
-    is_digit_converted: bool,
-
-    #[structopt(
-        name = "non-digits",
-        short = "D",
-        long,
-        help = "Converts any character which is not a Unicode decimal digit to \\D",
-        long_help = "Converts any character which is not a Unicode decimal digit to \\D.\n\n\
-                     Takes precedence over --non-words if both are set.\n\
-                     Non-digits which are also non-word characters are converted to \\D.\n\n\
-                     Takes precedence over --non-spaces if both are set.\n\
-                     Non-digits which are also non-space characters are converted to \\D.",
-        display_order = 2
-    )]
-    is_non_digit_converted: bool,
-
-    #[structopt(
-        name = "spaces",
-        short,
-        long,
-        help = "Converts any Unicode whitespace character to \\s",
-        long_help = "Converts any Unicode whitespace character to \\s.\n\n\
-                     Takes precedence over --non-digits if both are set.\n\
-                     Whitespace is converted to \\s, remaining non-digits to \\D.\n\n\
-                     Takes precedence over --non-words if both are set.\n\
-                     Whitespace is converted to \\s, remaining non-word characters to \\W.",
-        display_order = 3
-    )]
-    is_space_converted: bool,
-
-    #[structopt(
-        name = "non-spaces",
-        short = "S",
-        long,
-        help = "Converts any character which is not a Unicode whitespace character to \\S",
-        display_order = 4
-    )]
-    is_non_space_converted: bool,
-
-    #[structopt(
-        name = "words",
-        short,
-        long,
-        help = "Converts any Unicode word character to \\w",
-        long_help = "Converts any Unicode word character to \\w.\n\n\
-                     Takes precedence over --non-digits if both are set.\n\
-                     Word characters are converted to \\w, remaining non-digits to \\D.\n\n\
-                     Takes precedence over --non-spaces if both are set.\n\
-                     Word characters are converted to \\w, remaining non-whitespace to \\S.",
-        display_order = 5
-    )]
-    is_word_converted: bool,
-
-    #[structopt(
-        name = "non-words",
-        short = "W",
-        long,
-        help = "Converts any character which is not a Unicode word character to \\W",
-        long_help = "Converts any character which is not a Unicode word character to \\W.\n\n\
-                     Takes precedence over --non-spaces if both are set.\n\
-                     Non-word characters which are also non-whitespace are converted to \\W.",
-        display_order = 6
-    )]
-    is_non_word_converted: bool,
-
-    #[structopt(
-        name = "repetitions",
-        short,
-        long,
-        help = "Detects repeated non-overlapping substrings and\n\
-                converts them to {min,max} quantifier notation",
-        display_order = 7
-    )]
-    is_repetition_converted: bool,
-
-    #[structopt(
-        name = "escape",
-        short,
-        long,
-        help = "Replaces all non-ASCII characters with unicode escape sequences",
-        display_order = 8
-    )]
-    is_non_ascii_char_escaped: bool,
-
-    #[structopt(
-        name = "with-surrogates",
-        long,
-        requires = "escape",
-        help = "Converts astral code points to surrogate pairs if --escape is set",
-        display_order = 9
-    )]
-    is_astral_code_point_converted_to_surrogate: bool,
-
-    #[structopt(
-        name = "ignore-case",
-        short,
-        long,
-        help = "Performs case-insensitive matching, letters match both upper and lower case",
-        display_order = 10
-    )]
-    is_case_ignored: bool,
-
-    #[structopt(
-        name = "capture-groups",
-        short = "g",
-        long,
-        help = "Replaces non-capturing groups by capturing ones",
-        display_order = 11
-    )]
-    is_group_captured: bool,
-
-    #[structopt(
-        name = "verbose",
-        short = "x",
-        long,
-        help = "Produces a nicer looking regular expression in verbose mode",
-        display_order = 12
-    )]
-    is_verbose_mode_enabled: bool,
-
-    #[structopt(
-        name = "no-start-anchor",
-        long,
-        help = "Removes the caret anchor '^' from the resulting regular expression",
-        long_help = "Removes the caret anchor '^' from the resulting regular expression.\n\n\
-                     By default, the caret anchor is added to every generated regular\n\
-                     expression which guarantees that the expression matches the test cases\n\
-                     given as input only at the start of a string.\n\
-                     This flag removes the anchor, thereby allowing to match the test cases also\n\
-                     when they do not occur at the start of a string.",
-        display_order = 13
-    )]
-    is_caret_anchor_disabled: bool,
-
-    #[structopt(
-        name = "no-end-anchor",
-        long,
-        help = "Removes the dollar sign anchor '$' from the resulting regular expression",
-        long_help = "Removes the dollar sign anchor '$' from the resulting regular expression.\n\n\
-                     By default, the dollar sign anchor is added to every generated regular\n\
-                     expression which guarantees that the expression matches the test cases\n\
-                     given as input only at the end of a string.\n\
-                     This flag removes the anchor, thereby allowing to match the test cases also\n\
-                     when they do not occur at the end of a string.",
-        display_order = 14
-    )]
-    is_dollar_sign_anchor_disabled: bool,
-
-    #[structopt(
-        name = "no-anchors",
-        long,
-        help = "Removes the caret and dollar sign anchors from the resulting regular expression",
-        long_help = "Removes the caret and dollar sign anchors from the resulting regular expression.\n\n\
-                     By default, anchors are added to every generated regular expression\n\
-                     which guarantee that the expression exactly matches only the test cases\n\
-                     given as input and nothing else.\n\
-                     This flag removes the anchors, thereby allowing to match the test cases also\n\
-                     when they occur within a larger string that contains other content as well.",
-        display_order = 15
-    )]
-    are_anchors_disabled: bool,
-
-    #[structopt(
-        name = "colorize",
-        short,
-        long,
-        help = "Provides syntax highlighting for the resulting regular expression",
-        display_order = 16
-    )]
-    is_output_colorized: bool,
-
-    // --------------------
-    // OPTIONS
-    // --------------------
-    #[structopt(
+    /// Reads test cases on separate lines from a file.
+    ///
+    /// Lines may be ended with either a newline `\n` or a carriage return with a line feed `\r\n`.
+    /// The final line ending is optional.
+    #[clap(
         name = "file",
         value_name = "FILE",
         short,
         long,
-        parse(from_os_str),
-        required_unless = "input",
-        help = "Reads test cases on separate lines from a file",
-        long_help = "Reads test cases on separate lines from a file.\n\n\
-                     Lines may be ended with either a newline (`\\n`) or\n\
-                     a carriage return with a line feed (`\\r\\n`).\n\
-                     The final line ending is optional."
+        required_unless_present = "input",
+        value_parser,
+        help_heading = "INPUT"
     )]
     file_path: Option<PathBuf>,
 
-    #[structopt(
+    // --------------------
+    // DIGIT OPTIONS
+    // --------------------
+    /// Converts any Unicode decimal digit to \d.
+    ///
+    /// Takes precedence over --words if both are set.
+    /// Decimal digits are converted to \d, remaining word characters to \w.
+    ///
+    /// Takes precedence over --non-spaces if both are set.
+    /// Decimal digits are converted to \d, remaining non-space characters to \S.
+    #[clap(
+        name = "digits",
+        short,
+        long,
+        value_parser,
+        help_heading = "DIGIT OPTIONS"
+    )]
+    is_digit_converted: bool,
+
+    /// Converts any character which is not a Unicode decimal digit to \D.
+    ///
+    /// Takes precedence over --non-words if both are set.
+    /// Non-digits which are also non-word characters are converted to \D.
+    ///
+    /// Takes precedence over --non-spaces if both are set.
+    /// Non-digits which are also non-space characters are converted to \D.
+    #[clap(
+        name = "non-digits",
+        short = 'D',
+        long,
+        value_parser,
+        help_heading = "DIGIT OPTIONS"
+    )]
+    is_non_digit_converted: bool,
+
+    // --------------------
+    // WHITESPACE OPTIONS
+    // --------------------
+    /// Converts any Unicode whitespace character to \s.
+    ///
+    /// Takes precedence over --non-digits if both are set.
+    /// Whitespace is converted to \s, remaining non-digits to \D.
+    ///
+    /// Takes precedence over --non-words if both are set.
+    /// Whitespace is converted to \s, remaining non-word characters to \W.
+    #[clap(
+        name = "spaces",
+        short,
+        long,
+        value_parser,
+        help_heading = "WHITESPACE OPTIONS"
+    )]
+    is_space_converted: bool,
+
+    /// Converts any character which is not a Unicode whitespace character to \S
+    #[clap(
+        name = "non-spaces",
+        short = 'S',
+        long,
+        value_parser,
+        help_heading = "WHITESPACE OPTIONS"
+    )]
+    is_non_space_converted: bool,
+
+    // --------------------
+    // WORD OPTIONS
+    // --------------------
+    /// Converts any Unicode word character to \w.
+    ///
+    /// Takes precedence over --non-digits if both are set.
+    /// Word characters are converted to \w, remaining non-digits to \D.
+    ///
+    /// Takes precedence over --non-spaces if both are set.
+    /// Word characters are converted to \w, remaining non-whitespace to \S.
+    #[clap(
+        name = "words",
+        short,
+        long,
+        value_parser,
+        help_heading = "WORD OPTIONS"
+    )]
+    is_word_converted: bool,
+
+    /// Converts any character which is not a Unicode word character to \W.
+    ///
+    /// Takes precedence over --non-spaces if both are set.
+    /// Non-word characters which are also non-whitespace are converted to \W.
+    #[clap(
+        name = "non-words",
+        short = 'W',
+        long,
+        value_parser,
+        help_heading = "WORD OPTIONS"
+    )]
+    is_non_word_converted: bool,
+
+    // --------------------
+    // ESCAPING OPTIONS
+    // --------------------
+    /// Replaces all non-ASCII characters with unicode escape sequences.
+    #[clap(
+        name = "escape",
+        short,
+        long,
+        value_parser,
+        help_heading = "ESCAPING OPTIONS"
+    )]
+    is_non_ascii_char_escaped: bool,
+
+    /// Converts astral code points to surrogate pairs if --escape is set.
+    #[clap(
+        name = "with-surrogates",
+        long,
+        requires = "escape",
+        value_parser,
+        help_heading = "ESCAPING OPTIONS"
+    )]
+    is_astral_code_point_converted_to_surrogate: bool,
+
+    // --------------------
+    // REPETITION OPTIONS
+    // --------------------
+    /// Detects repeated non-overlapping substrings and converts them to {min,max} quantifier notation.
+    #[clap(
+        name = "repetitions",
+        short,
+        long,
+        value_parser,
+        help_heading = "REPETITION OPTIONS",
+        display_order = 1
+    )]
+    is_repetition_converted: bool,
+
+    /// Specifies the minimum quantity of substring repetitions to be converted if --repetitions is set.
+    #[clap(
         name = "min-repetitions",
         value_name = "QUANTITY",
         long,
-        default_value = "1",
-        validator = repetition_options_validator,
-        help = "Specifies the minimum quantity of substring repetitions\n\
-                to be converted if --repetitions is set"
+        default_value_t = 1,
+        value_parser = repetition_options_parser,
+        help_heading = "REPETITION OPTIONS"
     )]
     minimum_repetitions: u32,
 
-    #[structopt(
+    /// Specifies the minimum length a repeated substring must have
+    /// in order to be converted if --repetitions is set.
+    #[clap(
         name = "min-substring-length",
         value_name = "LENGTH",
         long,
-        default_value = "1",
-        validator = repetition_options_validator,
-        help = "Specifies the minimum length a repeated substring must have\n\
-                in order to be converted if --repetitions is set"
+        default_value_t = 1,
+        value_parser = repetition_options_parser,
+        help_heading = "REPETITION OPTIONS"
     )]
     minimum_substring_length: u32,
+
+    // --------------------
+    // ANCHOR OPTIONS
+    // --------------------
+    /// Removes the caret anchor `^` from the resulting regular expression.
+    ///
+    /// By default, the caret anchor is added to every generated regular expression
+    /// which guarantees that the expression matches the test cases
+    /// given as input only at the start of a string.
+    ///
+    /// This flag removes the anchor, thereby allowing to match the test cases
+    /// also when they do not occur at the start of a string.
+    #[clap(
+        name = "no-start-anchor",
+        long,
+        value_parser,
+        help_heading = "ANCHOR OPTIONS"
+    )]
+    is_caret_anchor_disabled: bool,
+
+    /// Removes the dollar sign anchor `$` from the resulting regular expression.
+    ///
+    /// By default, the dollar sign anchor is added to every generated regular expression
+    /// which guarantees that the expression matches the test cases given as input
+    /// only at the end of a string.
+    ///
+    /// This flag removes the anchor, thereby allowing to match the test cases
+    /// also when they do not occur at the end of a string.
+    #[clap(
+        name = "no-end-anchor",
+        long,
+        value_parser,
+        help_heading = "ANCHOR OPTIONS"
+    )]
+    is_dollar_sign_anchor_disabled: bool,
+
+    /// Removes the caret and dollar sign anchors from the resulting regular expression.
+    ///
+    /// By default, anchors are added to every generated regular expression
+    /// which guarantees that the expression exactly matches only the test cases given as input
+    /// and nothing else.
+    ///
+    /// This flag removes the anchors, thereby allowing to match the test cases
+    /// also when they occur within a larger string that contains other content as well.
+    #[clap(
+        name = "no-anchors",
+        long,
+        value_parser,
+        help_heading = "ANCHOR OPTIONS"
+    )]
+    are_anchors_disabled: bool,
+
+    // --------------------
+    // DISPLAY OPTIONS
+    // --------------------
+    /// Produces a nicer-looking regular expression in verbose mode.
+    #[clap(
+        name = "verbose",
+        short = 'x',
+        long,
+        value_parser,
+        help_heading = "DISPLAY OPTIONS",
+        display_order = 1
+    )]
+    is_verbose_mode_enabled: bool,
+
+    /// Provides syntax highlighting for the resulting regular expression.
+    #[clap(
+        name = "colorize",
+        short,
+        long,
+        value_parser,
+        help_heading = "DISPLAY OPTIONS"
+    )]
+    is_output_colorized: bool,
+
+    // ---------------------
+    // MISCELLANEOUS OPTIONS
+    // ---------------------
+    /// Performs case-insensitive matching, letters match both upper and lower case.
+    #[clap(
+        name = "ignore-case",
+        short,
+        long,
+        value_parser,
+        help_heading = "MISCELLANEOUS OPTIONS",
+        display_order = 1
+    )]
+    is_case_ignored: bool,
+
+    /// Replaces non-capturing groups with capturing ones.
+    #[clap(
+        name = "capture-groups",
+        short = 'g',
+        long,
+        value_parser,
+        help_heading = "MISCELLANEOUS OPTIONS",
+        display_order = 2
+    )]
+    is_group_captured: bool,
 }
 
 fn main() {
-    let cli = Cli::from_args();
+    let cli = Cli::parse();
     handle_input(&cli, obtain_input(&cli));
 }
 
@@ -400,11 +449,11 @@ fn handle_input(cli: &Cli, input: Result<Vec<String>, Error>) {
     }
 }
 
-fn repetition_options_validator(value: String) -> Result<(), String> {
+fn repetition_options_parser(value: &str) -> Result<u32, String> {
     match value.parse::<u32>() {
         Ok(parsed_value) => {
             if parsed_value > 0 {
-                Ok(())
+                Ok(parsed_value)
             } else {
                 Err(String::from("Value must not be zero"))
             }
