@@ -14,6 +14,8 @@
  * limitations under the License.
  */
 
+#![cfg(not(target_family = "wasm"))]
+
 use grex::RegExpBuilder;
 use indoc::indoc;
 use regex::Regex;
@@ -75,6 +77,7 @@ mod no_conversion {
             case(vec!["abxy", "cxy", "efgh"], "^(?:(?:ab|c)xy|efgh)$"),
             case(vec!["abxy", "efgh", "cxy"], "^(?:(?:ab|c)xy|efgh)$"),
             case(vec!["efgh", "abxy", "cxy"], "^(?:(?:ab|c)xy|efgh)$"),
+            case(vec!["aaacaac", "aac"], "^aa(?:acaa)?c$"),
             case(vec!["a", "ä", "o", "ö", "u", "ü"], "^[aouäöü]$"),
             case(vec!["y̆", "a", "z"], "^(?:y̆|[az])$"), // goal: "^[az]|y\\u{306}$"
             case(vec!["a", "b\n", "c"], "^(?:b\\n|[ac])$"),
@@ -90,7 +93,11 @@ mod no_conversion {
             case(vec!["My ♥ is yours.", "My 💩 is yours."], "^My [♥💩] is yours\\.$"),
             case(vec!["[\u{c3e}"], "^\\[\u{c3e}$"),
             case(vec!["\\\u{10376}"], "^\\\\\u{10376}$"),
-            case(vec!["I   ♥♥♥ 36 and ٣ and y̆y̆ and 💩💩."], "^I   ♥♥♥ 36 and ٣ and y̆y̆ and 💩💩\\.$")
+            case(vec!["I   ♥♥♥ 36 and ٣ and y̆y̆ and 💩💩."], "^I   ♥♥♥ 36 and ٣ and y̆y̆ and 💩💩\\.$"),
+            case(vec!["\u{890}\0"], "^\u{890}\0$"),
+            case(vec!["\u{890}\\0"], "^\u{890}\\\\0$"),
+            case(vec!["\u{890}\\\0"], "^\u{890}\\\\\0$"),
+            case(vec!["\u{890}\\\\0"], "^\u{890}\\\\\\\\0$")
         )]
         fn succeeds(test_cases: Vec<&str>, expected_output: &str) {
             let regexp = RegExpBuilder::from(&test_cases).build();
@@ -251,7 +258,46 @@ mod no_conversion {
                     4[,.]5
                   )
                 $"#
-            ))
+            )),
+            case(vec!["Ga", "G)"], indoc!(
+                r#"
+                (?x)
+                ^
+                  G[)a]
+                $"#
+            )),
+            case(vec!["aG", ")G"], indoc!(
+                r#"
+                (?x)
+                ^
+                  [)a]G
+                $"#
+            )),
+            case(vec!["Ga", "G)", "G("], indoc!(
+                r#"
+                (?x)
+                ^
+                  G[()a]
+                $"#
+            )),
+            case(vec!["aG", ")G", "(G"], indoc!(
+                r#"
+                (?x)
+                ^
+                  [()a]G
+                $"#
+            )),
+            case(vec!["aaacaac", "aac"], indoc!(
+                r#"
+                (?x)
+                ^
+                  aa
+                  (?:
+                    acaa
+                  )?
+                  c
+                $"#
+            )),
         )]
         fn succeeds_with_verbose_mode_option(test_cases: Vec<&str>, expected_output: &str) {
             let regexp = RegExpBuilder::from(&test_cases).with_verbose_mode().build();
@@ -306,21 +352,6 @@ mod no_conversion {
             assert_that_regexp_is_correct(regexp, expected_output, &test_cases);
             assert_that_regexp_matches_test_cases(expected_output, test_cases);
         }
-
-        #[rstest(test_cases, expected_output,
-            case(vec!["bab", "b", "cb", "bba"], "(?:b(?:ba|ab)?|cb)"),
-            case(vec!["a", "aba", "baaa", "aaab"], "(?:baaa|a(?:aab|ba)?)"),
-            case(vec!["a", "abab", "bbb", "aaac"], "(?:a(?:bab|aac)?|bbb)"),
-            case(
-                // https://github.com/pemistahl/grex/issues/31
-                vec!["agbhd", "eibcd", "egbcd", "fbjbf", "agbh", "eibc", "egbc", "ebc", "fbc", "cd", "f", "c", "abcd", "ebcd", "fbcd"], 
-                "(?:a(?:gbhd?|bcd)|e(?:ibcd?|gbcd?|bcd?)|f(?:b(?:jbf|cd?))?|cd?)")
-        )]
-        fn succeeds_without_anchors(test_cases: Vec<&str>, expected_output: &str) {
-            let regexp = RegExpBuilder::from(&test_cases).without_anchors().build();
-            assert_that_regexp_is_correct(regexp, expected_output, &test_cases);
-            assert_that_regexp_matches_test_cases(expected_output, test_cases);
-        }
     }
 
     mod repetition {
@@ -2036,31 +2067,45 @@ mod anchor_conversion {
         use super::*;
 
         #[rstest(test_cases, expected_output,
+            case(vec!["aaacaac", "aac"], "aa(?:acaa)?c$"),
             case(vec!["My ♥♥♥ and 💩💩 is yours."], "My ♥♥♥ and 💩💩 is yours\\.$"),
         )]
-        fn succeeds_with_no_start_anchor_option(test_cases: Vec<&str>, expected_output: &str) {
+        fn succeeds_without_start_anchor_option(test_cases: Vec<&str>, expected_output: &str) {
             let regexp = RegExpBuilder::from(&test_cases)
                 .without_start_anchor()
                 .build();
             assert_that_regexp_is_correct(regexp, expected_output, &test_cases);
+            assert_that_regexp_matches_test_cases(expected_output, test_cases);
         }
 
         #[rstest(test_cases, expected_output,
+            case(vec!["aaacaac", "aac"], "^aa(?:acaa)?c"),
             case(vec!["My ♥♥♥ and 💩💩 is yours."], "^My ♥♥♥ and 💩💩 is yours\\."),
         )]
-        fn succeeds_with_no_end_anchor_option(test_cases: Vec<&str>, expected_output: &str) {
+        fn succeeds_without_end_anchor_option(test_cases: Vec<&str>, expected_output: &str) {
             let regexp = RegExpBuilder::from(&test_cases)
                 .without_end_anchor()
                 .build();
             assert_that_regexp_is_correct(regexp, expected_output, &test_cases);
+            assert_that_regexp_matches_test_cases(expected_output, test_cases);
         }
 
         #[rstest(test_cases, expected_output,
+            case(vec!["bab", "b", "cb", "bba"], "(?:b(?:ba|ab)?|cb)"),
+            case(vec!["a", "aba", "baaa", "aaab"], "(?:baaa|a(?:aab|ba)?)"),
+            case(vec!["a", "abab", "bbb", "aaac"], "(?:a(?:bab|aac)?|bbb)"),
+            case(vec!["aaacaac", "aac"], "(?:aaacaac|aac)"), // "aa(?:acaa)?c" would not match "aaacaac"
             case(vec!["My ♥♥♥ and 💩💩 is yours."], "My ♥♥♥ and 💩💩 is yours\\."),
+            case(
+                // https://github.com/pemistahl/grex/issues/31
+                vec!["agbhd", "eibcd", "egbcd", "fbjbf", "agbh", "eibc", "egbc", "ebc", "fbc", "cd", "f", "c", "abcd", "ebcd", "fbcd"],
+                "(?:a(?:gbhd?|bcd)|e(?:ibcd?|gbcd?|bcd?)|f(?:b(?:jbf|cd?))?|cd?)"
+            )
         )]
-        fn succeeds_with_no_match_line_option(test_cases: Vec<&str>, expected_output: &str) {
+        fn succeeds_without_anchors(test_cases: Vec<&str>, expected_output: &str) {
             let regexp = RegExpBuilder::from(&test_cases).without_anchors().build();
             assert_that_regexp_is_correct(regexp, expected_output, &test_cases);
+            assert_that_regexp_matches_test_cases(expected_output, test_cases);
         }
     }
 
@@ -2075,7 +2120,7 @@ mod anchor_conversion {
                 $"#
             ))
         )]
-        fn succeeds_with_verbose_mode_and_no_start_anchor_option(
+        fn succeeds_with_verbose_mode_and_without_start_anchor_option(
             test_cases: Vec<&str>,
             expected_output: &str,
         ) {
@@ -2084,6 +2129,7 @@ mod anchor_conversion {
                 .without_start_anchor()
                 .build();
             assert_that_regexp_is_correct(regexp, expected_output, &test_cases);
+            assert_that_regexp_matches_test_cases(expected_output, test_cases);
         }
 
         #[rstest(test_cases, expected_output,
@@ -2094,7 +2140,7 @@ mod anchor_conversion {
                   My\ ♥♥♥\ and\ 💩💩\ is\ yours\."#
             ))
         )]
-        fn succeeds_with_verbose_mode_and_no_end_anchor_option(
+        fn succeeds_with_verbose_mode_and_without_end_anchor_option(
             test_cases: Vec<&str>,
             expected_output: &str,
         ) {
@@ -2103,16 +2149,26 @@ mod anchor_conversion {
                 .without_end_anchor()
                 .build();
             assert_that_regexp_is_correct(regexp, expected_output, &test_cases);
+            assert_that_regexp_matches_test_cases(expected_output, test_cases);
         }
 
         #[rstest(test_cases, expected_output,
+            case(vec!["aaacaac", "aac"], indoc!(
+                r#"
+                (?x)
+                  (?:
+                    aaacaac
+                    |
+                    aac
+                  )"#
+            )),
             case(vec!["My ♥♥♥ and 💩💩 is yours."], indoc!(
                 r#"
                 (?x)
                   My\ ♥♥♥\ and\ 💩💩\ is\ yours\."#
             ))
         )]
-        fn succeeds_with_verbose_mode_and_no_anchors_option(
+        fn succeeds_with_verbose_mode_and_without_anchors_option(
             test_cases: Vec<&str>,
             expected_output: &str,
         ) {
@@ -2121,6 +2177,7 @@ mod anchor_conversion {
                 .without_anchors()
                 .build();
             assert_that_regexp_is_correct(regexp, expected_output, &test_cases);
+            assert_that_regexp_matches_test_cases(expected_output, test_cases);
         }
     }
 }
@@ -2134,14 +2191,21 @@ fn assert_that_regexp_is_correct(regexp: String, expected_output: &str, test_cas
 }
 
 fn assert_that_regexp_matches_test_cases(expected_output: &str, test_cases: Vec<&str>) {
-    let re = Regex::new(expected_output).unwrap();
+    let regexp = Regex::new(expected_output).unwrap();
     for test_case in test_cases {
+        let substrings = regexp
+            .find_iter(test_case)
+            .map(|m| m.as_str())
+            .collect::<Vec<_>>();
+
         assert_eq!(
-            re.find_iter(test_case).count(),
+            substrings.len(),
             1,
-            "\n\n\"{}\" does not match regex {}\n\n",
+            "expression '{}' does not match test case '{}' entirely but {} of its substrings: {:?}",
+            expected_output,
             test_case,
-            expected_output
+            substrings.len(),
+            substrings
         );
     }
 }
